@@ -1,7 +1,7 @@
 /*
  * pftp -- sends files from host to host through free choosable ports
  *
- * Copyright (C) 1996, 1997, 1998 Ben Schluricke
+ * Copyright (C) 1996-1999 Ben Schluricke
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,8 +24,8 @@
  *
  *
  */
-#ifndef CONNECT
-#define CONNECT
+#ifndef PFTP_CONNECT_H
+#define PFTP_CONNECT_H
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
