@@ -1,7 +1,7 @@
 /*
  * pftp -- sends files from host to host through free choosable ports
  *
- * Copyright (C) 1996, 1997, 1998 Ben Schluricke
+ * Copyright (C) 1996-1999 Ben Schluricke
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,45 +24,29 @@
  *
  *
  */
-#ifndef MAIN
-#define MAIN
+#ifndef PFTP_MAIN_H
+#define PFTP_MAIN_H
 #include <sys/file.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-
-#define MAXCLIENTHOSTS       100 /* see main.c */
-#define MAXCLIENTS           5 /* see main.c */
-#define VERSION              "0.8.9"
-#define HUNAME               32767
-#define LONAME               1024
-#define SONAME               256
-#define BUFSIZE              32767
-#define DEFAULT_STDIN_BUFSIZ 4096
-#define MAXIMUM_UPLOAD_LIMIT 4096
-#define PFTP_NEW_DATA_MSG    "You have new data.\n"
-
 #include "status.h"
 extern struct status_type *statstr[MAXCLIENTS+1];
-
 #ifdef USE_POSIX_THREAD
 extern pthread_mutex_t interactiv;
 #endif
-extern short CHILDNUM; /* number of children currently running */
+extern int CHILDNUM; /* number of children currently running */
 extern short pftpnoroot; /* number of children currently running */
-extern int OUTPUT;
+extern long OUTPUT;
 extern int s;         /* Socket */
 extern FILE *slfp; /* pointer to stderr or server log file or NULL */
 extern volatile short _WINROWS_; /* Number of rows on the screen */
 extern volatile short _WINCOLS_; /* Number of columns on the screen */
 extern char *pftplog;  /* holds the name of the log file if any */
 extern char *_CLIENTHOSTNAME_[MAXCLIENTHOSTS]; /* specified client host names */
-extern char *filter[SONAME]; /* holds the name of the filter program and its arguments */
-extern char *_STDOUT_BUFFER_; /* buffers stdout data */
 extern char *client_destdir; /* Destination directory if sending to the daemon */
 extern char *data_subject; /* subject of data sent */
 extern char *data_info; /* holds info file about data sent */
-
 #endif
